﻿function global:Generate-MachineKey {
	param (
		[ValidateSet("AES", "DES", "3DES")]
		[string]$decryptionAlgorithm = 'AES',
		[ValidateSet("MD5", "SHA1", "HMACSHA256", "HMACSHA384", "HMACSHA512")]
		[string]$validationAlgorithm = 'SHA1'
	)
	function BinaryToHex {
		param($bytes)
		$builder = new-object System.Text.StringBuilder
		foreach ($b in $bytes) {
			$builder = $builder.AppendFormat([System.Globalization.CultureInfo]::InvariantCulture, "{0:X2}", $b)
		}
		$builder
	}
	switch ($decryptionAlgorithm) {
		"AES" { $decryptionObject = new-object System.Security.Cryptography.AesCryptoServiceProvider }
		"DES" { $decryptionObject = new-object System.Security.Cryptography.DESCryptoServiceProvider }
		"3DES" { $decryptionObject = new-object System.Security.Cryptography.TripleDESCryptoServiceProvider }
	}
	$decryptionObject.GenerateKey()
	$decryptionKey = BinaryToHex($decryptionObject.Key)
	$decryptionObject.Dispose()
	switch ($validationAlgorithm) {
		"MD5" { $validationObject = new-object System.Security.Cryptography.HMACMD5 }
		"SHA1" { $validationObject = new-object System.Security.Cryptography.HMACSHA1 }
		"HMACSHA256" { $validationObject = new-object System.Security.Cryptography.HMACSHA256 }
		"HMACSHA385" { $validationObject = new-object System.Security.Cryptography.HMACSHA384 }
		"HMACSHA512" { $validationObject = new-object System.Security.Cryptography.HMACSHA512 }
	}
	$validationKey = BinaryToHex($validationObject.Key)
	$validationObject.Dispose()
	[string]::Format([System.Globalization.CultureInfo]::InvariantCulture,
		"<machineKey decryption=`"{0}`" decryptionKey=`"{1}`" validation=`"{2}`" validationKey=`"{3}`" />",
		$decryptionAlgorithm.ToUpperInvariant(),                                                            
		$decryptionKey,
		$validationAlgorithm.ToUpperInvariant(),
		$validationKey)
}

function global:Set-MachineKeyToWebConfig {
	<# 
	.SYNOPSIS
		Задает значения для ключа machineKey в 
		Web.config приложения.
	.PARAMETER $ConfigPath
		Путь к конфигу приложения.
	.PARAMETER $MKey
		Елемент machineKey содержащий необходимые аттрибуты.
	#>
	param(
		[Parameter(Mandatory=$true,ValueFromPipelineByPropertyName=$true)]
		[string] $ConfigPath,
		[Parameter(Mandatory=$true,ValueFromPipelineByPropertyName=$true)]
		[xml] $MKey
	)
	Write-Host "Updating machineKey in config: [$ConfigPath]"
	if([string]::IsNullOrEmpty($ConfigPath)) { 
		Throw "Path to web.config is null or empty!";
	};
	if(-Not (Test-Path $ConfigPath)){
		Throw "Web config at path [$ConfigPath] was not found!";
	};
	$WebConfig = (Get-Content $ConfigPath) -as [Xml];
	if(-Not $WebConfig){
		Throw "[$ConfigPath] is not a valid xml!";           
	};
	$MachineKeyElements = $WebConfig.SelectSingleNode("//machineKey");
	if(-Not $MachineKeyElements) { 
		Throw "Element 'machineKey' was not found at [$ConfigPath]!" 
	};
	$mk = $MKey.machineKey;
	$MachineKeyElements.validationKey = $mk.validationKey;
	$MachineKeyElements.decryptionKey = $mk.decryptionKey;
	$MachineKeyElements.validation = $mk.validation;
	$WebConfig.Save($ConfigPath);
}


$webRootDirectory=$args[0]
	Write-Host "Updating site machineKey in [$webRootDirectory]"
	[XML]$MKey = Generate-MachineKey
	Set-MachineKeyToWebConfig -ConfigPath “$webRootDirectory\web.config” -MKey $MKey
	Set-MachineKeyToWebConfig -ConfigPath “$webRootDirectory\Terrasoft.WebApp\web.config” -MKey $MKey

